@@ -6,7 +6,7 @@ from isisdl.backend import sync_database
 from isisdl.backend.config import init_wizard, config_wizard
 from isisdl.backend.request_helper import CourseDownloader
 from isisdl.settings import is_first_time, is_static, forbidden_chars, has_ffmpeg, fstype, is_windows, working_dir_location, python_executable, is_macos, is_online
-from isisdl.utils import args, acquire_file_lock_or_exit, generate_error_message, install_latest_version, export_config, database_helper, config, migrate_database, Config
+from isisdl.utils import args, acquire_file_lock_or_exit, generate_error_message, install_latest_version, export_config, database_helper, config, migrate_database, Config, compare_download_diff
 from isisdl.version import __version__
 
 
@@ -34,6 +34,8 @@ It seems as if this is your first time executing isisdl. Welcome 💖
 I will guide you through a short configuration phase of about 5min.
 It is recommended that you read these options carefully.
 
+You may simply press the Enter key to accept the default.
+
 If you wish to re-configure me, run `isisdl --init` or `isisdl --config`.
 
 
@@ -92,6 +94,10 @@ Please press enter to continue.
         compress.main()
         sys.exit(0)
 
+    elif args.download_diff:
+        compare_download_diff()
+        sys.exit(0)
+
     # elif args.subscribe:
     #    print("Due to legal reasons, this is currently not supported. :(")
     #    sys.exit(0)
@@ -130,43 +136,59 @@ def main() -> None:
         generate_error_message(ex)
 
 
-# Feature discussion:
-#   Windows autorun
-#   Download of corrupted files
-
-
-# Main TODOS:
-#   Dynamic calculation of num threads for download
-#   fixing / bug hunting the compress
-
-
-# No size implies:
-#   More time conflicts in video files (File size from 10^9 → 10^4)
-#   No download status file size
-#   No sync database -> Not really since the videos names are sha-sums of the content
-
+#   Change Database → Version 1.4
+#       Make the size attribute not mandatory for a file
+#           → Logger looses the ability to report total number of bytes available
+#           → MediaContainer can't be compared with __gt__ → TODO: what does this cause? → Tests start failing in chop_down_size
+#           → Loose the ability to _always_ check if a file exists → if a file does not have a size, it is not downloaded
+#           → checking for conflict will be _way_ harder → TODO: How to solve? Videos at least have a video length size → Maybe use that?
+#           → DownloadStatus won't have an ETA
+#           → Syncing the Database will be of lower quality: Ignore all Documents and only restore the videos: Their name / url is the SHA-Sum of their content.
+#
+#       is_corrupted attribute change
+#           → Note the last time checked and retry the url based on an exponential backoff strategy
+#
+#       Remove the path attribute
+#           → Make the path of a file entirely dynamic and dependant on the configured state.
+#             This should lead to a seamless migration of the directory when changing it's name.
+#             This also enables easy renaming of courses and deselecting of the subdir option, assuming the directories have been moved accordingly.
+#
+#       No more json strings
+#           → Extra table for user configuration
+#
+#       Make storing your password more secure
+#           → Achieved by generating a random master password + random salt and store it in the database.
+#             This enables for greater security since 1 table is all that is needed in order to crack multiple passwords
+#
+#       Verify the database state on every startup by iterating over all files + sizes and raising an error if the size and checksum don't match
+#
+#       These changes imply the following
+#           → Don't check any URL's that are not extern
+#           →
 
 # TODO:
 #
-#   Files should only be in extern if they do not originate from `isis.tu-berlin.de`
-#
 #   Check how many urls requested multiple times
 #
+#   Calculate the number of threads to use dynamically
+#
+#   Check what isisdl is doing after building request cache → why is it so slow in valgrind?
+#
 #   Argcomplete
 #       https://pypi.org/project/argcomplete/
 #       qmk can also do this
+#   → Seems kinda hard / shitty to use
 #
 #   Compare program to check diffs against isia-tub
-#
-#   Refactor the config in database
-#       No more json strings
-#       When a file is corrupted also save the last time checked. Then use an exponential reevaluation algorithm to check if the urls are available again
+#       → More content
 #
 #   Throttling does not work when imposed via config ??
 
 
 # Maybe TODO
 #
+#   fixing / bug hunting the compress
+#
 #   Have a variable for posix and linux
 #
 #   Static builds from github runners
@@ -174,8 +196,5 @@ def main() -> None:
 #   Wiki tutorial for streaming
 
 
-# TODO with D-VA: How problematic is WZM?
-
-
 if __name__ == "__main__":
     main()
