@@ -39,7 +39,7 @@ from requests import Session
 
 from isisdl import settings
 from isisdl.backend.database_helper import DatabaseHelper
-from isisdl.settings import download_chunk_size, token_queue_download_refresh_rate, forbidden_chars, replace_dot_at_end_of_dir_name, force_filesystem, has_ffmpeg, fstype
+from isisdl.settings import download_chunk_size, token_queue_download_refresh_rate, forbidden_chars, replace_dot_at_end_of_dir_name, force_filesystem, has_ffmpeg, fstype, log_file_location
 from isisdl.settings import working_dir_location, is_windows, checksum_algorithm, checksum_num_bytes, example_config_file_location, config_dir_location, database_file_location, status_time, \
     discover_num_threads, status_progress_bar_resolution, download_progress_bar_resolution, config_file_location, is_first_time, is_autorun, parse_config_file, lock_file_location, \
     enable_lock, error_directory_location, systemd_dir_location, master_password, is_testing, systemd_timer_file_location, systemd_service_file_location, export_config_file_location, \
@@ -73,6 +73,9 @@ def get_args() -> argparse.Namespace:
     operations.add_argument("--stream", help="Launches isisdl in streaming mode. Will watch for file accesses and download only those files.", action="store_true")
     operations.add_argument("--update", help="Checks if an update is available and installs it.", action="store_true")
     operations.add_argument("--delete-bad-urls", help="Deletes all urls deemed to be \"bad\" - meaning there is no content.", action="store_true")
+    operations.add_argument("--download-diff", help="Checks if a given directory contains more / different content than downloaded from isisdl.", type=str)
+
+    # argcomplete.autocomplete(parser)
 
     if is_testing:
         return parser.parse_known_args()[0]
@@ -598,6 +601,20 @@ def check_github_for_version() -> Optional[Union[LegacyVersion, Version]]:
         return version.parse(found_version.group(1))
 
 
+def print_changelog_for_version(new_version: Union[LegacyVersion, Version]) -> None:
+    def version_check(v: str) -> bool:
+        return version.parse(__version__) < version.parse(v) <= new_version
+
+    current_version = version.parse(__version__)
+    assert not isinstance(current_version, LegacyVersion)
+    assert not isinstance(new_version, LegacyVersion)
+
+    for i in range(current_version.micro + 1, new_version.micro + 1):
+        con = requests.get(f"https://raw.githubusercontent.com/Emily3403/isisdl/main/src/isisdl/changelog/"
+                           f"{new_version.major}.{new_version.minor}/{new_version.major}.{new_version.minor}.{new_version.micro}.md")
+        print(con.text)
+
+
 def install_latest_version() -> None:
     if is_first_time:
         return
@@ -625,6 +642,7 @@ Please run `isisdl --init` to resolve this issue!
         return
 
     print(f"\nThere is a new version of isisdl available: {new_version} (current: {__version__}).")
+    print_changelog_for_version(new_version)
 
     if config.update_policy.startswith("notify") and not args.update:
         return
@@ -784,7 +802,7 @@ def sanitize_name(name: str, is_dir: bool) -> str:
         final = str_list
 
     # Folder names cannot end with a period in Windows ...
-    while replace_dot_at_end_of_dir_name and is_dir and final and (final[-1] == "." or final[-1] == " "):
+    while is_dir and replace_dot_at_end_of_dir_name and final and (final[-1] == "." or final[-1] == " "):
         final.pop()
 
     final_str = "".join(item for item in final if item not in forbidden_chars)
@@ -981,6 +999,52 @@ def calculate_local_checksum(filename: Path) -> str:
     return alg.hexdigest()
 
 
+def compare_download_diff() -> None:
+    # TODO: Make compressed videos work
+    from isisdl.backend.status import Status
+
+    def calc_checksum(file: Path, status: Status) -> str:
+        checksum = calculate_local_checksum(file)
+        status.add(file.stat().st_size)
+        return checksum
+
+    def calc_checksums(p: Path, extra_forbidden_paths: Set[Path]) -> Dict[str, Path]:
+        files = list(p.rglob("*"))
+        total_file_size = sum(it.stat().st_size for it in files)
+
+        with Status(f"Calculating checksums for {p}", total_file_size) as status:
+            if enable_multithread:
+                with ThreadPoolExecutor(os.cpu_count()) as ex:
+                    checksums = list(ex.map(calc_checksum, filter(lambda it: not it.is_dir() and it not in extra_forbidden_paths, files), repeat(status)))
+                    return {checksum: file for file, checksum in zip(filter(lambda it: not it.is_dir() and it not in extra_forbidden_paths, files), checksums)}
+            else:
+                return {calc_checksum(file, status): file for file in filter(lambda it: not it.is_dir() and it not in extra_forbidden_paths, files)}
+
+    forbidden_files = {path(database_file_location), path(lock_file_location), path(log_file_location)}
+
+    other_path = Path(args.download_diff)
+
+    s = time.perf_counter()
+    isisdl_checksums = calc_checksums(path(), forbidden_files)
+    print(f"This step took {time.perf_counter() - s:.3f} s")
+
+    s = time.perf_counter()
+    other_checksums = calc_checksums(other_path, set())
+    print(f"This step took {time.perf_counter() - s:.3f} s")
+
+    with open(path("diff.txt"), "w") as f:
+
+        f.write(f"Files downloaded which are in {path()} but not in {other_path}:\n")
+        for checksum in set(isisdl_checksums) - set(other_checksums):
+            f.write(f"{isisdl_checksums[checksum]}\n")
+
+        f.write(f"\n\nFiles downloaded which are in {other_path} but not in {path()}:\n")
+        for checksum in set(other_checksums) - set(isisdl_checksums):
+            f.write(f"{other_checksums[checksum]}\n")
+
+    print(f"The diff was written to {path('diff.txt')}")
+
+
 def subscribe_to_all_courses() -> None:
     from isisdl.backend.request_helper import RequestHelper
     from isisdl.backend.crypt import get_credentials
@@ -1296,6 +1360,7 @@ class DownloadThrottler(Thread):
         return int((self.download_rate * 1024 ** 2) // download_chunk_size * self.refresh_rate) or 1
 
 
+# TODO: Add link type and store that in the database + reference to checksum?<
 class MediaType(enum.Enum):
     document = 1
     extern = 2
